/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;

import com.redhat.rhevm.api.mock.util.SimpleQueryEvaluator;
import com.redhat.rhevm.api.model.DataCenter;
import com.redhat.rhevm.api.model.Network;
import com.redhat.rhevm.api.model.Networks;
import com.redhat.rhevm.api.model.NetworkStatus;
import com.redhat.rhevm.api.resource.NetworkResource;
import com.redhat.rhevm.api.resource.NetworksResource;
import com.redhat.rhevm.api.common.util.StatusUtils;

import static com.redhat.rhevm.api.mock.resource.AbstractMockResource.allocateId;


public class MockNetworksResource extends AbstractMockQueryableResource<Network> implements NetworksResource {

    private static Map<String, MockNetworkResource> networks =
        Collections.synchronizedMap(new HashMap<String, MockNetworkResource>());

    public MockNetworksResource() {
        super(new SimpleQueryEvaluator<Network>());
    }

    public void populate() {
        synchronized (networks) {
            while (networks.size() < 4) {
                MockNetworkResource resource = new MockNetworkResource(allocateId(Network.class), getExecutor(), this);
                resource.getModel().setName("network" + resource.getModel().getId());
                resource.getModel().setStatus((networks.size() % 2) == 0 ? StatusUtils.create(NetworkStatus.OPERATIONAL) : StatusUtils.create(NetworkStatus.NON_OPERATIONAL));
                DataCenter dataCenter = new DataCenter();
                dataCenter.setId(resource.getModel().getId());
                resource.getModel().setDataCenter(dataCenter);
                networks.put(resource.getModel().getId(), resource);
            }
        }
    }

    @Override
    public Networks list() {
        Networks ret = new Networks();

        for (MockNetworkResource network : networks.values()) {
            if (filter(network.getModel(), getUriInfo(), Network.class)) {
                ret.getNetworks().add(network.addLinks());
            }
        }

        return ret;
    }

    @Override
    public Response add(Network network) {
        MockNetworkResource resource = new MockNetworkResource(allocateId(Network.class), getExecutor(), this);

        resource.updateModel(network);

        String id = resource.getId();
        networks.put(id, resource);

        UriBuilder uriBuilder = getUriInfo().getAbsolutePathBuilder().path(id);

        network = resource.addLinks();

        return Response.created(uriBuilder.build()).entity(network).build();
    }

    @Override
    public void remove(String id) {
        networks.remove(id);
    }

    @Override
    public NetworkResource getNetworkSubResource(String id) {
        return networks.get(id);
    }
}
