/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.concurrent.Executor;

import javax.ws.rs.core.Response;

import com.redhat.rhevm.api.model.Action;
import com.redhat.rhevm.api.model.Template;
import com.redhat.rhevm.api.resource.AssignedPermissionsResource;
import com.redhat.rhevm.api.resource.CreationResource;
import com.redhat.rhevm.api.resource.DevicesResource;
import com.redhat.rhevm.api.resource.TemplateResource;
import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.common.util.JAXBHelper;
import com.redhat.rhevm.api.common.util.LinkHelper;


public class MockTemplateResource extends AbstractMockResource<Template> implements TemplateResource {

    /**
     * Package-protected ctor, never needs to be instantiated by JAX-RS framework.
     *
     * @param template     encapsulated template
     * @param executor executor used for asynchronous actions
     */
    MockTemplateResource(String id, Executor executor, UriInfoProvider uriProvider) {
        super(id, executor, uriProvider);
    }

    // FIXME: this needs to be atomic
    public void updateModel(Template template) {
        // update writable fields only
        if (template.isSetName()) {
            getModel().setName(template.getName());
        }
        if (template.isSetDescription()) {
            getModel().setDescription(template.getDescription());
        }
    }

    public Template addLinks() {
        return LinkHelper.addLinks(getUriInfo(), JAXBHelper.clone("template", Template.class, getModel()));
    }

    @Override
    public Template get() {
        return addLinks();
    }

    @Override
    public Template update(Template template) {
        validateUpdate(template);
        updateModel(template);
        return addLinks();
    }

    @Override
    public Response export(Action action) {
        return null;
    }

    @Override public DevicesResource             getCdRomsResource()      { return null; }
    @Override public DevicesResource             getDisksResource()       { return null; }
    @Override public DevicesResource             getNicsResource()        { return null; }
    @Override public AssignedPermissionsResource getPermissionsResource() { return null; }
    @Override public CreationResource            getCreationSubresource(String ids) { return null; }
}
