/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;
import javax.ws.rs.core.UriInfo;

import com.redhat.rhevm.api.mock.util.SimpleQueryEvaluator;
import com.redhat.rhevm.api.model.User;
import com.redhat.rhevm.api.model.Users;
import com.redhat.rhevm.api.resource.UserResource;
import com.redhat.rhevm.api.resource.UsersResource;

import static com.redhat.rhevm.api.mock.resource.AbstractMockResource.allocateId;


public class MockUsersResource extends AbstractMockQueryableResource<User> implements UsersResource {

    private static Map<String, MockUserResource> users =
        Collections.synchronizedMap(new HashMap<String, MockUserResource>());

    public MockUsersResource() {
        super(new SimpleQueryEvaluator<User>());
    }

    public void populate() {
        synchronized (users) {
            while (users.size() < 2) {
                MockUserResource resource = new MockUserResource(allocateId(User.class), getExecutor(), this);
                User user = resource.getModel();
                user.setName("user" + resource.getModel().getId());
                user.setUserName("user_" + resource.getModel().getId());
                users.put(user.getId(), resource);
            }
        }
    }

    @Override
    public Users list() {
        Users ret = new Users();

        for (MockUserResource user : users.values()) {
            UriInfo info = getUriInfo();
            if (filter(user.getModel(), info, User.class)) {
                ret.getUsers().add(user.addLinks());
            }
        }

        return ret;
    }

    @Override
    public Response add(User user) {
        MockUserResource resource = new MockUserResource(allocateId(User.class), getExecutor(), this);

        resource.updateModel(user);

        String id = resource.getId();
        users.put(id, resource);

        UriBuilder uriBuilder = getUriInfo().getAbsolutePathBuilder().path(id);

        user = resource.addLinks();

        return Response.created(uriBuilder.build()).entity(user).build();
    }

    @Override
    public void remove(String id) {
        users.remove(id);
    }

    @Override
    public UserResource getUserSubResource(String id) {
        return users.get(id);
    }
}
