/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.concurrent.Executor;

import javax.ws.rs.core.Response;

import com.redhat.rhevm.api.model.Action;
import com.redhat.rhevm.api.model.Cluster;
import com.redhat.rhevm.api.model.VM;
import com.redhat.rhevm.api.resource.AssignedPermissionsResource;
import com.redhat.rhevm.api.resource.AssignedTagsResource;
import com.redhat.rhevm.api.resource.CreationResource;
import com.redhat.rhevm.api.resource.DevicesResource;
import com.redhat.rhevm.api.resource.SnapshotsResource;
import com.redhat.rhevm.api.resource.StatisticsResource;
import com.redhat.rhevm.api.resource.VmResource;
import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.common.util.JAXBHelper;
import com.redhat.rhevm.api.common.util.LinkHelper;
import com.redhat.rhevm.api.mock.model.MockVmStatus;


public class MockVmResource extends AbstractMockResource<VM> implements VmResource {
    /* FIXME: would like to do:
     * private @Context UriInfo uriInfo;
     */

    @SuppressWarnings("unused")
    private MockVmStatus status;

    /**
     * Package-protected ctor, never needs to be instantiated by JAX-RS framework.
     * @param executor executor used for asynchronous actions
     * @param vm       encapsulated VM
     */
    public MockVmResource(String id, Executor executor, UriInfoProvider uriProvider) {
        super(id, executor, uriProvider);
    }

    // FIXME: this needs to be atomic
    public void updateModel(VM vm) {
        // update writable fields only
        if (vm.isSetName()) {
            getModel().setName(vm.getName());
        }
        if (vm.isSetDescription()) {
            getModel().setDescription(vm.getDescription());
        }
        if (vm.isSetCluster()) {
            Cluster cluster = new Cluster();
            cluster.setId(vm.getCluster().getId());
            getModel().setCluster(cluster);
        }
    }

    public VM addLinks() {
        return LinkHelper.addLinks(getUriInfo(), JAXBHelper.clone("vm", VM.class, getModel()));
    }

    /* FIXME: kill uriInfo param, make href auto-generated? */
    @Override
    public VM get() {
        return addLinks();
    }

    @Override
    public VM update(VM vm) {
        validateUpdate(vm);
        updateModel(vm);
        return addLinks();
    }

    protected String[] getStrictlyImmutable() {
        return addStrictlyImmutable("type");
    }

    @Override
    public Response start(Action action) {
        return doAction(getUriInfo(), new VmStatusSetter(action, MockVmStatus.UP));
    }


    @Override
    public Response stop(Action action) {
        return doAction(getUriInfo(), new VmStatusSetter(action, MockVmStatus.DOWN));

    }

    @Override
    public Response shutdown(Action action) {
        return doAction(getUriInfo(), new VmStatusSetter(action, MockVmStatus.DOWN));
    }

    @Override
    public Response suspend(Action action) {
        return doAction(getUriInfo(), new DoNothingTask(action));
    }

    @Override
    public Response detach(Action action) {
        return doAction(getUriInfo(), new DoNothingTask(action));
    }

    @Override
    public Response migrate(Action action) {
        return doAction(getUriInfo(), new DoNothingTask(action));
    }

    @Override
    public Response export(Action action) {
        return null;
    }

    @Override
    public Response move(Action action) {
        return null;
    }


    @Override
    public Response ticket(Action action) {
        return doAction(getUriInfo(), new DoNothingTask(action));
    }

    private class VmStatusSetter extends AbstractActionTask {
        private MockVmStatus status;
        VmStatusSetter(Action action, MockVmStatus status) {
            super(action, "VM status failed with {0}");
            this.status = status;
        }
        public void execute() {
            if (status.equals(MockVmResource.this.status)) {
                throw new IllegalStateException("VM status already: " + status);
            }
            MockVmResource.this.status = status;
        }
    }

    @Override public DevicesResource             getCdRomsResource()      { return null; }
    @Override public DevicesResource             getDisksResource()       { return null; }
    @Override public DevicesResource             getNicsResource()        { return null; }
    @Override public SnapshotsResource           getSnapshotsResource()   { return null; }
    @Override public AssignedPermissionsResource getPermissionsResource() { return null; }
    @Override public AssignedTagsResource        getTagsResource()        { return null; }
    @Override public CreationResource            getCreationSubresource(String ids) { return null; }
    @Override public StatisticsResource          getStatisticsResource()  { return null; }
}
